#ifndef STOCHASTIC_NODE_H_
#define STOCHASTIC_NODE_H_

//class StochasticNode;
#include <graph/Node.h>
#include <sarray/SArray.h>

#include <vector>

class Distribution;

class StochasticNode : public Node {
    Distribution const *_dist;
    std::vector<SArray const *> _parameters;

    /* Forbid copying of Stochastic Nodes */
    StochasticNode(StochasticNode const &orig);
    StochasticNode &operator=(StochasticNode const &rhs);
 public:

    //StochasticNode();
    StochasticNode(Index const &dim = Index(1));
    ~StochasticNode();

    bool isBounded() const;
    SArray const *lowerBound();
    SArray const *upperBound();
    void setBounds(Node *lBound, Node *uBound);
    Distribution const *distribution() const;
    void setDistribution(Distribution const *dist);
    void setParameters(std::vector<Node*> const &parameters);
    std::vector<SArray const *> const &parameters() const;
    double logDensity();

    void forwardSample();
};

StochasticNode *asStochastic(Node *node);
bool isStochastic(Node const *node);

#endif /* STOCHASTIC_NODE_H_ */
