#ifndef COMPILER_H_
#define COMPILER_H_

#include <model/LogicalFactory.h>
#include <model/CounterTab.h>

#include <map>

class ParseTree;
class Graph;
class SymTab;
class Index;
class FuncTab;
class DistTab;

class Compiler {
  Graph &_graph;
  SymTab &_symtab;
  CounterTab _countertab;
  FuncTab const &_functab;
  DistTab const &_disttab;
  std::map<std::string, SArray> const &_data_table;    
  LogicalFactory _logicalfactory;

  Node *Param(ParseTree const *t);
  bool indexExpression(ParseTree const *t, long &value);
  bool constantExpression(ParseTree const *p, double &value);
  Range VariableSubsetRange(ParseTree const *var);
  Range CounterRange(ParseTree const *var);
  Node* VarGetNode(ParseTree const *var);
  Node* VarGetParameter(ParseTree const *var);
  bool getRange(std::vector<ParseTree*> const &range_list, 
		Range const &default_range, Range &result);

  void writeStochastic(ParseTree const *node);
  void writeLogical(ParseTree const *node);
  Node* getLogicalParameter(ParseTree const *t);
  void getLogicalParameterVector(ParseTree const *t,
			      std::vector<Node*> &parents);
  void makeSubsetIndex(ParseTree const *p, 
		       std::vector<SubSetIndex> &index,
		       Index const &default_index);
  Node * getSubSetNode(ParseTree const *var);
  Node * getMixtureNode(ParseTree const *var);
public:
  /**
   * Constructor
   * @param graph Graph to be created. It should initially be empty
   * @param symtab Symbol table to be created. It should initially be empty
   * @param functab Function table to look up functions by name
   * @param disttab Distribution table to look up distributions by name
   * @param datatab Data table. This is required since some constant
   * expressions in the bugs language may depend on data values.
   */
  Compiler(Graph &graph, SymTab &symtab,
	   FuncTab const &functab, DistTab const &disttab,
	   std::map<std::string, SArray> const &data_table);
  void declareVariables(std::vector<ParseTree*> const &pvariables);
  void allocateNodes(ParseTree const *prelations);
  void fillConstants();
  void writeRelations(ParseTree const *prelations);
  void writeValuesToGraph(std::map<std::string, SArray> const &data_table,
			  bool isdata);
  void collectNodes();
};


#endif /* COMPILER_H_ */
