#ifndef CONJUGATE_SAMPLER_H_
#define CONJUGATE_SAMPLER_H_

#include <sampler/GibbsSampler.h>

class StochasticNode;
class LogicalNode;

enum ConjugateDist {
  BERN, BETA, BIN, CAT, CHISQ, DEXP, DIRCH, EXP, GAMMA,
  LNORM, LOGIS, MNORM, MULTI, NEGBIN, NORM, PAR, POIS,
  T, UNIF, WEIB, WISH, OTHERDIST
};

enum ConjugateOp {
  ADD, SUBTRACT, UMINUS, MULTIPLY, DIVIDE, IDENTITY, INPROD, OTHEROP
};

/**
 * Base class for conjugate samplers.  This class provides only
 * two static functions that simplify the calculation to help
 * the canSample() member function of conjugate samplers
 */
class ConjugateSampler : public GibbsSampler
{
public:
  ConjugateSampler(StochasticNode *snode);
  /**
   * Returns the distribution of stochastic node as member of
   * the ConjugateDist enumeration
   */
  static ConjugateDist getDist(StochasticNode const *snode);
  /** 
   * Returns the function of a logical node as a member of
   * the ConjugateOp enumeration.  
   */
  static ConjugateOp getOp(LogicalNode const *lnode);
};


#endif /* CONJUGATE_SAMPLER_H_ */
