#ifndef INDEX_H_
#define INDEX_H_

/**
 * An Index behaves like a one-dimensional array of long integers, with
 * boundary checking.  Its principal uses are representing the dimension
 * of an SArray, and indexing the elements of an SArray.
 * @short array of long integers
 */
class Index {
    unsigned int _length;
    long *_index;
 public:
    /**
     * Constructs an index of given length. All elements of the index
     * are set to 1.
     */
    Index(unsigned int length);
    /**
     * Copy constructor
     */
    Index(Index const &index);
    ~Index();

    /**
     * Assignment operator
     */
    Index &operator=(Index const &rhs);

    /**
     * Number of elements in the index
     */
    unsigned int length() const;
    /**
     * Subscripting operator. Allows an index to behave like an array.
     * @param i index to access (starting from 0). 
     * @exception out_of_range
     */
    long &operator[](unsigned int i);
    long operator[](unsigned int i) const;
    /**
     * Test for equality of two indices.
     */
    bool operator==(Index const &other) const;
    /**
     * Inequality operator.
     */
    bool operator!=(Index const &other) const;
};

/**
 * Comparison function for indices based on strcmp. Can be used
 * to create a complete ordering among a set of unique indices.
 * @return -1, 1, or 0 depending on whether i1 comes before or after i2,
 *         or is equal to i2.
 */
int indexComp(Index const &i1, Index const &i2);

#endif /* INDEX_H_ */
