#ifndef LEFT_RANGE_ITERATOR_H_
#define LEFT_RANGE_ITERATOR_H_

#include <sarray/Index.h>
#include <sarray/Range.h>

/**
 * A LeftRangeIterator is an index that traverses - via the increment
 * operator - a given range, moving the left hand index fastest.
 * @see Index Range
 */
class LeftRangeIterator : public Index {
    const Range _range;
 public:
    /**
     * Constructor. The initial value of a LeftRangeIterator is
     * the lower limit of the range argument.
     * @param range. Range to traverse
     */
    LeftRangeIterator(Range const &range);
    /**
     * Assignment operator. A LeftRangeIterator can be set to any value
     * inside the range.
     * @exception range_error
     */
    LeftRangeIterator &operator=(Index const &index);
    /*
    void reset();
    */
    /**
     * Test to see if the LeftRangeIterator is at the upper limit 
     * of the range. For use in for() loops
     */
    bool atEnd() const;
    /**
     * Prefix increment operator. There is no postfix increment
     * operator for efficiency reasons
     * @return reference to self
     */
    LeftRangeIterator &operator++();
};

#endif /* LEFT_RANGE_ITERATOR_H_ */
