#ifndef RIGHT_RANGE_ITERATOR_H_
#define RIGHT_RANGE_ITERATOR_H_

#include <sarray/Range.h>
#include <sarray/Index.h>

/**
 * A RightRangeIterator is an index that traverses - via the increment
 * operator - a given range, moving the right hand index fastest.
 * @see Index Range
 */
class RightRangeIterator : public Index {
	const Range _range;
 public:
	/**
	 * Constructor. The initial value of a RightRangeIterator is
	 * the lower limit of the range argument.
	 * @param range. Range to traverse
	 */
	RightRangeIterator(Range const &range);
	/**
	 * Assignment operator. A RightRangeIterator can be set to any value
	 * inside the range.
	 * @exception range_error
	 */
	RightRangeIterator &operator=(Index const &index);
	//void reset();
	/**
	 * Test to see if the RightRangeIterator is at the upper limit 
	 * of the range. For use in for() loops.
	 */
	bool atEnd() const;
	/**
	 * Prefix increment operator. There is no postfix increment
	 * operator for efficiency reasons.
	 * @return reference to self
	 */
	RightRangeIterator &operator++();
};

#endif /* RIGHT_RANGE_ITERATOR_H_ */
